package net.maunium.bukkit.Mau1v1;

import java.util.List;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

public class Mau1v1Manager {
	private Mau1v1 plugin;
	public Mau1v1Manager(Mau1v1 plugin){
		this.plugin = plugin;
	}
	
	protected void challenge(Player challenger, Player challenged){
		if(challenged.isOnline()){
			plugin.setMetadata(challenged, Mau1v1.CHALLENGER, challenger);
			challenger.sendMessage(plugin.stag + "Challenged " + challenged.getName() + " to an 1v1 battle.");
			challenged.sendMessage(plugin.stag + challenger.getName() + " has challenged you to a 1v1 battle.");
			challenged.sendMessage(plugin.stag + "Use /1v1 accept or /1v1 deny.");
		} else {
			challenger.sendMessage(plugin.errtag + "The player " + challenged.getName() + " is not online.");
		}
	}
	protected void accept(Player challenger, Player challenged){
		if(plugin.hasMetadata(challenger, Mau1v1.OPPONENT)){
			challenged.sendMessage(plugin.errtag + challenger.getName() + " is already in 1v1!");
			return;
		}
		challenger.sendMessage(plugin.stag + challenged.getName() + " has accepted your challenge!");
		challenged.sendMessage(plugin.stag + "You have accepted " + challenger.getName() + "'s challenge!");
		plugin.removeMetadata(challenged, Mau1v1.CHALLENGER);
		plugin.man.enterBattle(challenger, challenged);
	}
	protected void deny(Player challenger, Player challenged){
		challenger.sendMessage(plugin.stag + challenged.getName() + " has denied your challenge.");
		challenged.sendMessage(plugin.stag + "You're such a noob. Denying challenges.. :3");
		plugin.removeMetadata(challenged, Mau1v1.CHALLENGER);
	}
	
	protected void death(Player loser, Player wiener){
		try{
			duelEnd(loser, wiener, true);
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	protected void quit(Player loser, Player wiener){
		World w = loser.getWorld();
		Location l = loser.getLocation();
		for(ItemStack is : loser.getInventory().getContents()){
			if(is != null)
				w.dropItem(l, is);
		}
		loser.getInventory().clear();
		
		try{
			duelEnd(loser, wiener, false);
		} catch (Exception e){
			e.printStackTrace();
		}
		loser.setHealth(0);
	}
	private void duelEnd(Player loser, Player wiener, boolean kill){
		wiener.sendMessage(plugin.stag + "You're the wiener!");
		if(kill) loser.sendMessage(plugin.stag + "You lost!");
		
		Object o = plugin.getMetadata(wiener, Mau1v1.ARENA).value();
		if(!(o instanceof Arena)){
			throw new ClassCastException(wiener.getName() + "'s " + Mau1v1.ARENA + " metadata value is not an instance of Arena!");
		}
		final Arena a = (Arena) o;
		
		if(kill)
			Bukkit.broadcastMessage(plugin.stag + wiener.getName() + " killed " + loser.getName() + " at 1v1 " + a.getName() + "!");
		else
			Bukkit.broadcastMessage(plugin.stag + loser.getName() + " has noobed out of a 1v1 battle against " + wiener.getName() + "!");
		
		List<Entity> entities = wiener.getNearbyEntities(16, 8, 16);
		for(Entity e : entities){
			if(e.getType().equals(EntityType.DROPPED_ITEM)){
				e.remove();
			}
		}
		
		wiener.teleport(wiener.getWorld().getSpawnLocation());
		a.setInUse(false);
		
		plugin.removeMetadata(wiener, Mau1v1.OPPONENT);
		plugin.removeMetadata(wiener, Mau1v1.PVPING);
		plugin.removeMetadata(wiener, Mau1v1.ARENA);
		
		plugin.removeMetadata(loser, Mau1v1.OPPONENT);
		plugin.removeMetadata(loser, Mau1v1.PVPING);
		plugin.removeMetadata(loser, Mau1v1.ARENA);
	}
	
	protected void enterBattle(final Player p1, final Player p2){
		plugin.setMetadata(p1, Mau1v1.OPPONENT, p2);
		plugin.setMetadata(p2, Mau1v1.OPPONENT, p1);
		
		Runnable r = new Runnable(){
			@Override
			public void run(){
				for(Arena a : plugin.arenas){
					if(!a.isReady()) continue;
					if(a.isInUse()) continue;
					plugin.setMetadata(p1, Mau1v1.ARENA, a);
					plugin.setMetadata(p2, Mau1v1.ARENA, a);
					a.setInUse(true);
					
					plugin.setMetadata(p1, Mau1v1.NOMOVE, true);
					plugin.setMetadata(p2, Mau1v1.NOMOVE, true);
					p1.teleport(a.getSpawn1());
					p2.teleport(a.getSpawn2());
					
					p1.sendMessage("Battle starting in 5 seconds...");
					p2.sendMessage("Battle starting in 5 seconds...");
					try { Thread.sleep(1000); }
					catch (InterruptedException e) { e.printStackTrace(); }
					p1.sendMessage("4...");
					p2.sendMessage("4...");
					try { Thread.sleep(1000); }
					catch (InterruptedException e) { e.printStackTrace(); }
					p1.sendMessage("3...");
					p2.sendMessage("3...");
					try { Thread.sleep(1000); }
					catch (InterruptedException e) { e.printStackTrace(); }
					p1.sendMessage("2...");
					p2.sendMessage("2...");
					try { Thread.sleep(1000); }
					catch (InterruptedException e) { e.printStackTrace(); }
					p1.sendMessage("1...");
					p2.sendMessage("1...");
					try { Thread.sleep(1000); }
					catch (InterruptedException e) { e.printStackTrace(); }
					plugin.removeMetadata(p1, Mau1v1.NOMOVE);
					plugin.removeMetadata(p2, Mau1v1.NOMOVE);
					plugin.setMetadata(p1, Mau1v1.PVPING, true);
					plugin.setMetadata(p2, Mau1v1.PVPING, true);
					p1.sendMessage("GO!");
					p2.sendMessage("GO!");
					return;
				}
				plugin.removeMetadata(p1, Mau1v1.OPPONENT);
				plugin.removeMetadata(p1, Mau1v1.ARENA);
				plugin.removeMetadata(p2, Mau1v1.OPPONENT);
				plugin.removeMetadata(p2, Mau1v1.ARENA);
				
				p1.sendMessage(plugin.errtag + "No free arenas found!");
				p2.sendMessage(plugin.errtag + "No free arenas found!");
			}
		};
		plugin.getServer().getScheduler().runTaskAsynchronously(plugin, r);
	}
}
