package net.maunium.bukkit.MauBukLib;

import java.io.File;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import java.util.logging.Logger;

import lib.PatPeter.SQLibrary.Database;
import lib.PatPeter.SQLibrary.SQLite;

import org.bukkit.Location;

public class PlayerData {
	private Database sql;
	
	public PlayerData(File dataFolder) {
		sql = new SQLite(Logger.getLogger("Minecraft"), "[MauBukLib] ", dataFolder.getAbsolutePath(), "players", ".maudb");
		if (!sql.isOpen()) sql.open();
		try {
			sql.query("CREATE TABLE Players(UUID varchar(25) PRIMARY KEY,Username TEXT NOT NULL,IP TEXT NOT NULL,LastLogin INTEGER NOT NULL,Location TEXT NOT NULL);");
		} catch (SQLException e) {}
	}
	
	public void close() {
		sql.close();
	}
	
	public ResultSet setEntry(UUID uuid, String username, String ip, Location l) throws SQLException {
		return sql.query("INSERT OR REPLACE INTO Players VALUES('" + uuid.toString() + "','" + username + "','" + ip + "','" + System.currentTimeMillis()
				+ "','" + MauUtils.toString(l) + "');");
	}
	
	public ResultSet updateTime(UUID uuid) throws SQLException {
		return sql.query("UPDATE Players SET LastLogin='" + System.currentTimeMillis() + "' WHERE UUID='" + uuid.toString() + "';");
	}
	
	public ResultSet updateLocation(UUID uuid, Location l) throws SQLException {
		return sql.query("UPDATE Players SET Location='" + MauUtils.toString(l) + "' WHERE UUID='" + uuid.toString() + "';");
	}
	
	/**
	 * @return A list of all the usernames of the players that previously logged in from the given IP.
	 */
	public List<String> getPlayersByIP(String ip) {
		try {
			ResultSet rs = sql.query("SELECT Username FROM Players WHERE IP='" + ip + "'");
			List<String> players = new ArrayList<String>();
			while (rs.next())
				players.add(rs.getString("Username"));
			return players;
		} catch (Throwable e) {
			return Arrays.asList();
		}
	}
	
	/**
	 * @return The name of the player with the given UUID.
	 */
	public String getPlayerByUUID(UUID uuid) {
		try {
			ResultSet rs = sql.query("SELECT Username FROM Players WHERE UUID='" + uuid.toString() + "'");
			return rs.getString("Username");
		} catch (Throwable e) {
			return "";
		}
	}
	
	/**
	 * @return Get all the UUIDs that have been used along with the given name.
	 */
	public List<UUID> getUUIDsByName(String name) {
		try {
			ResultSet rs = sql.query("SELECT UUID FROM Players WHERE Username='" + name + "'");
			List<UUID> players = new ArrayList<UUID>();
			while (rs.next())
				players.add(UUID.fromString(rs.getString("UUID")));
			return players;
		} catch (Throwable e) {
			return Arrays.asList();
		}
	}
	
	/**
	 * @return A list of all the UUIDs of the players that previously logged in from the given IP.
	 */
	public List<UUID> getUUIDsByIP(String ip) {
		try {
			ResultSet rs = sql.query("SELECT UUID FROM Players WHERE IP='" + ip + "'");
			List<UUID> players = new ArrayList<UUID>();
			while (rs.next())
				players.add(UUID.fromString(rs.getString("UUID")));
			return players;
		} catch (Throwable e) {
			return Arrays.asList();
		}
	}
	
	/**
	 * @return The previous IP from where the player with the given UUID has logged in.
	 */
	public String getIPByUUID(UUID uuid) {
		String s = uuid.toString();
		try {
			ResultSet rs = sql.query("SELECT IP FROM Players WHERE UUID='" + s + "'");
			return rs.getString("IP");
		} catch (Throwable e) {
			return "";
		}
	}
	
	/**
	 * @return List of all the IPs that have the given username.
	 * @deprecated Quite useless, since this will only return more than one IP if the UUID of the player has changed.
	 */
	@Deprecated
	public List<String> getIPsByName(String name) {
		try {
			ResultSet rs = sql.query("SELECT IP FROM Players WHERE Username='" + name + "'");
			List<String> players = new ArrayList<String>();
			while (rs.next())
				players.add(rs.getString("IP"));
			return players;
		} catch (Throwable e) {
			return Arrays.asList();
		}
	}
	
	/**
	 * @return The previous time when the player with the given UUID logged in.
	 */
	public long getLastLoginByUUID(UUID uuid) {
		try {
			ResultSet rs = sql.query("SELECT LastLogin FROM Players WHERE UUID='" + uuid.toString() + "'");
			return rs.getLong("LastLogin");
		} catch (Throwable e) {
			return 0;
		}
	}
	
	/**
	 * @return The login/logout location of the player with the given UUID.
	 */
	public Location getLocationByUUID(UUID uuid) {
		try {
			ResultSet rs = sql.query("SELECT Location FROM Players WHERE UUID='" + uuid.toString() + "'");
			String s = rs.getString("Location");
			return MauUtils.parseLocation(s);
		} catch (Throwable e) {
			return null;
		}
	}
}
