package net.maunium.bukkit.MauLaunchers;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.entity.Player;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.util.Vector;

@SerializableAs("MauLauncher")
public class Launcher implements Serializable, ConfigurationSerializable {
	private static final long serialVersionUID = 293150001;
	private static final String metaX = "MauLauncher-X_velocity", metaY = "MauLauncher-Y_velocity", metaZ = "MauLauncher-Z_velocity";
	private static MauLaunchers plugin;
	private int x, y, z;
	private double velX, velY, velZ;
	private UUID world;
	
	public Launcher(Block b, Vector v){
		this.x = b.getX();
		this.y = b.getY();
		this.z = b.getZ();
		this.velX = v.getX();
		this.velY = v.getY();
		this.velZ = v.getZ();
		this.world = b.getWorld().getUID();
	}
	
	public Launcher(int x, int y, int z, Vector v, UUID world){
		this.x = x;
		this.y = y;
		this.z = z;
		this.velX = v.getX();
		this.velY = v.getY();
		this.velZ = v.getZ();
		this.world = world;
	}
	
	public Launcher(Block b, double velX, double velY, double velZ){
		this.x = b.getX();
		this.y = b.getY();
		this.z = b.getZ();
		this.velX = velX;
		this.velY = velY;
		this.velZ = velZ;
		this.world = b.getWorld().getUID();
	}
	
	public Launcher(int x, int y, int z, double velX, double velY, double velZ, UUID world) {
		this.x = x;
		this.y = y;
		this.z = z;
		this.velX = velX;
		this.velY = velY;
		this.velZ = velZ;
		this.world = world;
	}
	
	public Launcher(Map<String, Object> serialized) {
		this.x = (int) serialized.get("pos-x");
		this.y = (int) serialized.get("pos-y");
		this.z = (int) serialized.get("pos-z");
		this.velX = (double) serialized.get("vel-x");
		this.velY = (double) serialized.get("vel-y");
		this.velZ = (double) serialized.get("vel-z");
		this.world = UUID.fromString(serialized.get("world").toString());
	}
	
	public void deactivate() {
		deactivate(getBlock());
	}
	
	public static void deactivate(Block b){
		b.removeMetadata(metaX, plugin);
		b.removeMetadata(metaY, plugin);
		b.removeMetadata(metaZ, plugin);
	}
	
	public boolean activate() {
		Block b = getBlock();
		boolean x = isLauncher(b);
		b.setMetadata(metaX, new FixedMetadataValue(plugin, velX));
		b.setMetadata(metaY, new FixedMetadataValue(plugin, velY));
		b.setMetadata(metaZ, new FixedMetadataValue(plugin, velZ));
		return x;
	}
	
	@Override
	public Map<String, Object> serialize() {
		Map<String, Object> rtrn = new HashMap<String, Object>();
		rtrn.put("pos-x", x);
		rtrn.put("pos-y", y);
		rtrn.put("pos-z", z);
		rtrn.put("vel-x", velX);
		rtrn.put("vel-y", velY);
		rtrn.put("vel-z", velZ);
		rtrn.put("world", world.toString());
		return rtrn;
	}
	
	@Override
	public String toString(){
		return "Launcher with vector " + velX + ", " + velY + ", " + velZ + " @ " + x + ", " + y + ", " + z + " in world " + getBlock().getWorld().getName();
	}
	
	public double getVelX(){
		return velX;
	}
	public double getVelY(){
		return velY;
	}
	public double getVelZ(){
		return velZ;
	}
	
	public Block getBlock(){
		World w = Bukkit.getServer().getWorld(world);
		Block b = w.getBlockAt(x, y, z);
		return b;
	}
	
	public static boolean isLauncher(Block b){
		return b.hasMetadata(metaX) && b.hasMetadata(metaY) && b.hasMetadata(metaZ);
	}
	
	public static void launchPlayer(Player p, Block b) {
		try {
			if (isLauncher(b)) {
				double x = getMeta(b, metaX).asDouble();
				double y = getMeta(b, metaY).asDouble();
				double z = getMeta(b, metaZ).asDouble();
				p.setVelocity(new Vector(x, y, z));
				p.setFallDistance(0.0F);
				p.setMetadata(plugin.fallMeta, new FixedMetadataValue(plugin, true));
			}
		} catch (Throwable t) {}
	}
	
	private static MetadataValue getMeta(Block b, String tag) {
		for (MetadataValue mv : b.getMetadata(tag))
			if (mv.getOwningPlugin().equals(plugin)) return mv;
		return null;
	}
	
	public static Launcher deserialize(Map<String, Object> serialized) {
		return new Launcher(serialized);
	}
	
	public static Launcher valueOf(Map<String, Object> serialized) {
		return new Launcher(serialized);
	}
	
	public static void setHost(MauLaunchers ml) {
		if (plugin == null) plugin = ml;
	}
}
