package net.maunium.bukkit.MauPerms;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Arrays;

import org.bukkit.configuration.serialization.ConfigurationSerialization;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.plugin.ServicePriority;
import org.bukkit.plugin.java.JavaPlugin;

import net.maunium.bukkit.MauBukLib.I18n;
import net.maunium.bukkit.MauPerms.Commands.CommandMauperms;
import net.maunium.bukkit.MauPerms.Commands.CommandNick;
import net.maunium.bukkit.MauPerms.Listeners.ChatListener;
import net.maunium.bukkit.MauPerms.Listeners.JQListener;
import net.maunium.bukkit.MauPerms.Listeners.PreCommandListener;
import net.maunium.bukkit.MauPerms.Permissions.GroupPerms;
import net.maunium.bukkit.MauPerms.Permissions.PermissionManager;
import net.maunium.bukkit.MauPerms.Permissions.PlayerPerms;
import net.maunium.bukkit.MauPerms.Permissions.VaultPermsBridge;

import net.milkbowl.vault.permission.Permission;

public class MauPerms extends JavaPlugin {
	public String version, name, author, errtag, stag;
	private boolean ksd = false;
	private I18n i18n;
	
	@Override
	public void onEnable() {
		try {
			int ksd = getKSD("vilku124-maunium");
			if (ksd != 0) {
				this.ksd = true;
				this.getPluginLoader().disablePlugin(this);
				this.setEnabled(false);
				return;
			}
			long st = System.currentTimeMillis();
			version = this.getDescription().getVersion();
			name = this.getDescription().getName();
			author = this.getDescription().getAuthors().get(0);
			stag = "2[a" + name + "2]7 ";
			errtag = "4[c" + name + "4]c ";
			
			ConfigurationSerialization.registerClass(PlayerPerms.class);
			ConfigurationSerialization.registerClass(GroupPerms.class);
			
			this.saveDefaultConfig();
			this.saveResource("en_US.lang", true);
			this.saveResource("fi_FI.lang", true);
			try {
				i18n = new I18n(this.getDataFolder(), getConfig().getString("language"));
			} catch (IOException e) {
				e.printStackTrace();
			}
			PermissionManager.initialize(this);
			regList(new ChatListener());
			regList(new JQListener());
			regList(new PreCommandListener(this));
			this.getCommand("nick").setExecutor(new CommandNick(this));
			this.getCommand("mauperms").setExecutor(new CommandMauperms(this));
			
			if (getServer().getPluginManager().getPlugin("Vault") != null) {
				getLogger().info("Loading Vault permissions bridge.");
				try {
					VaultPermsBridge vpb = new VaultPermsBridge();
					getServer().getServicesManager().register(Permission.class, vpb, this, ServicePriority.Highest);
					getLogger().info("Loaded Vault permissions bridge.");
				} catch (Exception e) {
					getLogger().severe("Failed to load Vault permissions bridge.");
					e.printStackTrace();
				}
			} else {
				getLogger().warning("Vault not found. Not loading bridge.");
			}
			
			for (Player p : getServer().getOnlinePlayers()) {
				PermissionManager.getInstance().loadPermissions(p.getUniqueId());
			}
			
			int et = (int) (System.currentTimeMillis() - st);
			getLogger().info(name + " v" + version + " by " + author + " enabled in " + et + "ms.");
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	@Override
	public void onDisable() {
		if (ksd) return;
		long st = System.currentTimeMillis();
		
		try {
			for (Player p : getServer().getOnlinePlayers()) {
				PermissionManager.getInstance().unloadPermissions(p.getUniqueId());
			}
			PermissionManager.getInstance().saveGroups();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " disabled in " + et + "ms.");
	}
	
	public void regList(Listener l) {
		this.getServer().getPluginManager().registerEvents(l, this);
	}
	
	public static int getKSD(String appname) {
		Socket s;
		try {
			s = new Socket("maunium.net", 29351);
		} catch (UnknownHostException e1) {
			return -1;
		} catch (IOException e1) {
			return -2;
		}
		try {
			byte[] b = appname.getBytes("UTF-8");
			s.getOutputStream().write(1);
			s.getOutputStream().write(b.length);
			s.getOutputStream().write(b);
		} catch (UnsupportedEncodingException e) {
			close(s);
			return -3;
		} catch (IOException e) {
			close(s);
			return -4;
		}
		int response;
		try {
			response = s.getInputStream().read();
		} catch (IOException e) {
			close(s);
			return -5;
		}
		close(s);
		return response;
	}
	
	private static void close(Socket s) {
		try {
			s.close();
		} catch (IOException e) {}
	}
	
	public String format(String s, Object... args) {
		if (i18n != null) {
			return i18n.format(s, args);
		} else {
			return s + " -- " + Arrays.toString(args);
		}
	}
}
