package net.maunium.bukkit.MauPerms.Permissions;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;

@SerializableAs(value = "MauGroupPermissions")
public class GroupPerms implements Serializable, ConfigurationSerializable {
	static final long serialVersionUID = 29309000L;
	private List<String> inheritances;
	private Map<String, Boolean> permissions;
	private String chatFormat;
	
	public GroupPerms() {
		this.inheritances = new ArrayList<String>();
		this.permissions = new HashMap<String, Boolean>();
		this.chatFormat = "";
	}
	
	public GroupPerms(Map<String, Object> map) {
		this.inheritances = new ArrayList<String>();
		this.permissions = new HashMap<String, Boolean>();
		Object inheritances = map.get("inheritances");
		Object perms = map.get("permissions");
		Object chatformat = map.get("chat-format");
		if (inheritances instanceof List && perms instanceof Map && chatformat instanceof String) {
			this.chatFormat = (String) chatformat;
			for (Object o : (List<?>) inheritances) {
				if (o instanceof String) this.inheritances.add((String) o);
			}
			for (Entry<?, ?> e : ((Map<?, ?>) perms).entrySet()) {
				if (e.getKey() instanceof String && e.getValue() instanceof Boolean) permissions.put((String) e.getKey(), (Boolean) e.getValue());
			}
		}
	}
	
	public String getChatFormat() {
		return chatFormat;
	}
	
	public void setChatFormat(String s) {
		this.chatFormat = s;
	}
	
	public boolean addInheritance(String group) {
		if (!inheritances.contains(group)) {
			inheritances.add(group);
			return true;
		} else {
			return false;
		}
	}
	
	public boolean removeInheritance(String group) {
		if (inheritances.contains(group)) {
			inheritances.remove(group);
			return true;
		} else {
			return false;
		}
	}
	
	public boolean addPermission(String permission, boolean enabled) {
		permission = permission.toLowerCase();
		if (!permissions.containsKey(permission)) {
			permissions.put(permission, enabled);
			return true;
		} else if (!permissions.get(permission).equals(enabled)) {
			permissions.put(permission, enabled);
			return true;
		} else {
			return false;
		}
	}
	
	public boolean hasPermission(String permission) {
		permission = permission.toLowerCase();
		if (permission.startsWith("-")) {
			if (permissions.containsKey(permission.substring(1))) {
				if (!permissions.get(permission.substring(1))) { return true; }
			}
		} else if (permissions.containsKey(permission)) {
			if (permissions.get(permission)) { return true; }
		}
		return false;
	}
	
	public boolean removePermission(String permission) {
		permission = permission.toLowerCase();
		if (permissions.containsKey(permission)) {
			permissions.remove(permission);
			return true;
		} else {
			return false;
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<String> getInheritances() {
		return (List<String>) ((ArrayList<String>) inheritances).clone();
	}
	
	@SuppressWarnings("unchecked")
	public Map<String, Boolean> getPermissions() {
		return (Map<String, Boolean>) ((HashMap<String, Boolean>) permissions).clone();
	}
	
	public Map<String, Boolean> getPermissionsWithInheritances() {
		Map<String, Boolean> rtrn = getPermissions();
		for (String s : inheritances) {
			rtrn.putAll(PermissionManager.getInstance().getGroup(s).getPermissionsWithInheritances());
		}
		return rtrn;
	}
	
	public static GroupPerms deserialize(Map<String, Object> map) {
		return new GroupPerms(map);
	}
	
	public static GroupPerms valueOf(Map<String, Object> map) {
		return new GroupPerms(map);
	}
	
	@Override
	public Map<String, Object> serialize() {
		Map<String, Object> rtrn = new HashMap<String, Object>();
		rtrn.put("inheritances", inheritances);
		rtrn.put("permissions", permissions);
		rtrn.put("chat-format", chatFormat);
		return rtrn;
	}
}
