package net.maunium.bukkit.MauPerms.Permissions;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.UUID;
import java.util.logging.Level;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;

import net.maunium.bukkit.MauPerms.MauPerms;

public class PermissionManager {
	private MauPerms plugin;
	private Map<UUID, PlayerPerms> players;
	private Map<String, GroupPerms> groups;
	private String defaultGroup = "";
	private FileConfiguration permissions;
	private File permFile;
	private static PermissionManager instance;
	
	private PermissionManager(MauPerms plugin) {
		this.plugin = plugin;
		this.players = new HashMap<UUID, PlayerPerms>();
		if (!plugin.getConfig().getString("permission-location").equals("local")) {
			permFile = new File(plugin.getConfig().getString("permissions-location"));
		} else {
			permFile = new File(plugin.getDataFolder(), "permissions.yml");
		}
		reloadPermConfig();
		instance = this;
		groups = new HashMap<String, GroupPerms>();
		try {
			plugin.getLogger().info("Loading groups...");
			if (permissions.contains("groups")) {
				plugin.getLogger().finer("Path Exists.");
				for (String s : permissions.getConfigurationSection("groups").getValues(false).keySet()) {
					Object o = permissions.get("groups." + s);
					if (o instanceof GroupPerms) {
						GroupPerms gp = (GroupPerms) o;
						groups.put(s, gp);
						plugin.getLogger().finer("Added group " + s);
					} else {
						plugin.getLogger().warning("Group " + s + " was not an instance of GroupPerms in the config.");
						plugin.getLogger().warning("It did not get added to the group list " + o);
					}
					
				}
			} else {
				plugin.getLogger().finer("No groups found!");
			}
		} catch (Exception e) {
			plugin.getLogger().severe("Error while trying to load groups...");
			e.printStackTrace();
		}
		defaultGroup = permissions.getString("default-group");
	}
	
	public void saveGroups() {
		plugin.getLogger().info("Saving groups...");
		for (Entry<String, GroupPerms> e : groups.entrySet()) {
			try {
				plugin.getLogger().info("Saving group " + e.getKey());
				permissions.set("groups." + e.getKey(), e.getValue());
			} catch (Exception ex) {
				plugin.getLogger().severe("Error while setting group " + e.getKey() + " to config. ");
				ex.printStackTrace();
				continue;
			}
		}
		permissions.set("default-group", defaultGroup);
		
		try {
			permissions.save(permFile);
		} catch (IOException ex) {
			plugin.getLogger().log(Level.SEVERE, "Could not save config to " + permFile, ex);
		}
	}
	
	public static void initialize(MauPerms plugin) {
		new PermissionManager(plugin);
	}
	
	public static PermissionManager getInstance() {
		return instance;
	}
	
	public String getDefaultGroup() {
		return defaultGroup;
	}
	
	public void setDefaultGroup(String defaultGroup) {
		this.defaultGroup = defaultGroup.toLowerCase();
	}
	
	public Set<String> getGroups() {
		return groups.keySet();
	}
	
	public PlayerPerms loadPermissions(UUID p) {
		PlayerPerms pp = (PlayerPerms) permissions.get("players." + p);
		if (pp == null) pp = new PlayerPerms();
		
		Player ppp = plugin.getServer().getPlayer(p);
		if (ppp != null) if (ppp.isOnline()){
			if (!containsGroup(pp.getGroup())) {
				pp.setGroup(getDefaultGroup());
			}
			players.put(p, pp);
			pp.attach(plugin, ppp);
		}
		return pp;
	}
	
	public void savePermissions(UUID p, PlayerPerms pp){
		permissions.set("players." + p, pp);
		Player ppp = plugin.getServer().getPlayer(p);
		if(ppp != null) if (ppp.isOnline()) pp.detach(plugin, ppp);
	}
	
	public void unloadPermissions(UUID p) {
		savePermissions(p, getPlayer(p));
		if(players.containsKey(p)) players.remove(p);
	}
	
	public PlayerPerms getPlayer(UUID uuid) {
		if (players.containsKey(uuid)) return players.get(uuid);
		else return loadPermissions(uuid);
	}
	
	public void setPlayer(UUID uuid, PlayerPerms value) {
		if (players.containsKey(uuid)) {
			players.put(uuid, value);
			value.refresh();
		} else {
			players.put(uuid, value);
		}
	}
	
	public GroupPerms getGroup(String name) {
		for (Entry<String, GroupPerms> e : groups.entrySet()) {
			if (e.getKey().equalsIgnoreCase(name)) { return e.getValue(); }
		}
		return null;
	}
	
	public boolean containsGroup(String name) {
		for (String s : groups.keySet()) {
			if (s.equalsIgnoreCase(name)) return true;
		}
		return false;
	}
	
	public boolean removeGroup(String name) {
		if (containsGroup(name)) {
			groups.remove(fixCase(name));
			for (Entry<UUID, PlayerPerms> e2 : players.entrySet()) {
				PlayerPerms pp = e2.getValue();
				if (!containsGroup(pp.getGroup())) {
					pp.setGroup(getDefaultGroup());
					players.put(e2.getKey(), pp);
				}
			}
			return true;
		}
		return false;
	}
	
	public String fixCase(String s) {
		for (String s2 : getGroups()) {
			if (s.equalsIgnoreCase(s2)) return s2;
		}
		return null;
	}
	
	public void setGroup(String name, GroupPerms value) {
		if (containsGroup(name)) {
			groups.put(fixCase(name), value);
		} else {
			groups.put(name, value);
		}
	}
	
	public void reloadPermConfig() {
		if (permFile.exists()) permissions = YamlConfiguration.loadConfiguration(permFile);
		else permissions = new YamlConfiguration();
	}
	
	public FileConfiguration getPerms() {
		return permissions;
	}
}
