package net.maunium.bukkit.MauPerms.Permissions;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.bukkit.Bukkit;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.entity.Player;
import org.bukkit.permissions.Permissible;
import org.bukkit.permissions.Permission;
import org.bukkit.permissions.PermissionAttachment;
import org.bukkit.plugin.Plugin;

import net.maunium.bukkit.MauBukLib.ChatFormatter;

@SerializableAs(value = "MauPlayerPermissions")
public class PlayerPerms implements Serializable, ConfigurationSerializable {
	static final long serialVersionUID = 29308000L;
	private String group;
	private List<String> subGroups;
	private Map<String, Boolean> permissions;
	private String nick;
	private String chatFormat;
	private transient PermissionAttachment pa;
	
	public PlayerPerms() {
		group = PermissionManager.getInstance().getDefaultGroup();
		subGroups = new ArrayList<String>();
		permissions = new HashMap<String, Boolean>();
		nick = "";
		chatFormat = "";
	}
	
	public boolean addSubgroup(String s) {
		return this.subGroups.add(s);
	}
	
	public boolean removeSubgroup(String s) {
		return this.subGroups.remove(s);
	}
	
	public boolean hasCustomNick() {
		return !nick.trim().isEmpty();
	}
	
	public String getGroup() {
		return group;
	}
	
	public void setGroup(String group) {
		this.group = group;
	}
	
	public String getNick() {
		return nick;
	}
	
	public void setNick(String nick) {
		this.nick = nick;
	}
	
	@SuppressWarnings("unchecked")
	public List<String> getSubgroups() {
		return (List<String>) ((ArrayList<String>) subGroups).clone();
	}
	
	@SuppressWarnings("unchecked")
	public Map<String, Boolean> getPermissions() {
		return (Map<String, Boolean>) ((HashMap<String, Boolean>) permissions).clone();
	}
	
	public void setChatFormat(String chatFormat) {
		this.chatFormat = chatFormat;
	}
	
	public String getChatFormat() {
		if (hasCustomChatFormat()) return chatFormat;
		else return PermissionManager.getInstance().getGroup(group).getChatFormat();
	}
	
	public boolean hasCustomChatFormat() {
		return !chatFormat.isEmpty();
	}
	
	public boolean addPermission(String permission, boolean enabled) {
		permission = permission.toLowerCase();
		if (!permissions.containsKey(permission)) {
			permissions.put(permission, enabled);
			return true;
		} else {
			return false;
		}
	}
	
	public boolean removePermission(String permission) {
		permission = permission.toLowerCase();
		if (permissions.containsKey(permission)) {
			permissions.remove(permission);
			return true;
		} else {
			return false;
		}
	}
	
	public PlayerPerms(Map<String, Object> map) {
		subGroups = new ArrayList<String>();
		permissions = new HashMap<String, Boolean>();
		Object group = map.get("group");
		Object subgroups = map.get("subgroups");
		Object perms = map.get("permissions");
		Object nick = map.get("nick");
		Object chatformat = map.get("chat-format");
		if (group instanceof String && subgroups instanceof List<?> && perms instanceof Map<?, ?> && nick instanceof String && chatformat instanceof String) {
			this.group = (String) group;
			this.nick = ChatFormatter.full((String) nick);
			this.chatFormat = (String) chatformat;
			for (Object o : (List<?>) subgroups) {
				if (o instanceof String) {
					this.subGroups.add((String) o);
				}
			}
			for (Entry<?, ?> e : ((Map<?, ?>) perms).entrySet()) {
				if (e.getKey() instanceof String && e.getValue() instanceof Boolean) {
					permissions.put((String) e.getKey(), (Boolean) e.getValue());
				}
			}
		}
	}
	
	public void refresh() {
		if (pa != null) {
			detach(pa.getPlugin(), pa.getPermissible());
			attach(pa.getPlugin(), pa.getPermissible());
		}
	}
	
	private boolean checkStar(String perm, boolean enabled, PermissionAttachment pa) {
		if (perm.endsWith("*") && Bukkit.getServer().getPluginManager().getPermission(perm) == null) {
			if (perm.equals("*")) {
				for (Permission pp : Bukkit.getServer().getPluginManager().getPermissions()) {
					pa.setPermission(pp, enabled);
				}
				return true;
			} else {
				for (Permission pp : Bukkit.getServer().getPluginManager().getPermissions()) {
					if (pp.getName().startsWith(perm.substring(perm.length() - 2, perm.length()))) pa.setPermission(pp, enabled);
				}
				return true;
			}
		}
		return false;
	}
	
	public void attach(Plugin plugin, Permissible p) {
		pa = p.addAttachment(plugin);
		for (Entry<String, Boolean> e : permissions.entrySet()) {
			if (checkStar(e.getKey(), e.getValue(), pa)) continue;
			pa.setPermission(e.getKey(), e.getValue());
		}
		if (!group.isEmpty() && PermissionManager.getInstance().containsGroup(group)) {
			GroupPerms gp = PermissionManager.getInstance().getGroup(group);
			for (Entry<String, Boolean> e : gp.getPermissionsWithInheritances().entrySet()) {
				if (checkStar(e.getKey(), e.getValue(), pa)) continue;
				pa.setPermission(e.getKey(), e.getValue());
			}
		}
		for (String s : subGroups) {
			if (!s.isEmpty() || !PermissionManager.getInstance().containsGroup(s)) {
				for (Entry<String, Boolean> e : PermissionManager.getInstance().getGroup(s).getPermissions().entrySet()) {
					if (checkStar(e.getKey(), e.getValue(), pa)) continue;
					pa.setPermission(e.getKey(), e.getValue());
				}
			}
		}
		if (p instanceof Player) {
			Player pp = (Player) p;
			if(nick.isEmpty()) nick = pp.getName();
			pp.setDisplayName(nick);
			if (plugin.getConfig().getBoolean("change-tablistname")) {
				if (nick.length() > 16) {
					nick = nick.substring(0, 16);
					pp.setPlayerListName(nick);
				}
			}
		}
	}
	
	public void detach(Plugin plugin, Permissible p) {
		if (pa != null) {
			p.removeAttachment(pa);
		}
		if (p instanceof Player) {
			Player pp = (Player) p;
			pp.setDisplayName(pp.getName());
			if (plugin.getConfig().getBoolean("change-tablistname")) {
				if (nick.length() > 16) {
					nick = nick.substring(0, 16);
					pp.setPlayerListName(nick);
				}
			}
		}
	}
	
	@Override
	public Map<String, Object> serialize() {
		Map<String, Object> rtrn = new HashMap<String, Object>();
		rtrn.put("group", group);
		rtrn.put("subgroups", subGroups);
		rtrn.put("permissions", permissions);
		rtrn.put("nick", ChatFormatter.reverse(nick));
		rtrn.put("chat-format", chatFormat);
		return rtrn;
	}
	
	public static PlayerPerms deserialize(Map<String, Object> map) {
		return new PlayerPerms(map);
	}
	
	public static PlayerPerms valueOf(Map<String, Object> map) {
		return new PlayerPerms(map);
	}
}
