package net.maunium.bukkit.MauPerms.Permissions;

import java.util.Collection;

import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import net.milkbowl.vault.permission.Permission;

@SuppressWarnings("deprecation")
public class VaultPermsBridge extends Permission {
	
	@Override
	public String getName() {
		return "MauPerms";
	}
	
	@Override
	public boolean isEnabled() {
		return true;
	}
	
	@Override
	public boolean hasSuperPermsCompat() {
		return true;
	}
	
	@Override
	public boolean hasGroupSupport() {
		return true;
	}
	
	@Override
	public boolean has(CommandSender sender, String permission) {
		if (sender instanceof Player) return has((Player) sender, permission);
		else return sender.hasPermission(permission);
	}
	
	@Override
	public boolean has(Player player, String permission) {
		permission = permission.toLowerCase();
		PlayerPerms pp = PermissionManager.getInstance().getPlayer(player.getUniqueId());
		if(pp.getPermissions().containsKey("*") && pp.getPermissions().get("*")) return true;
		if (permission.startsWith("-")) {
			if (pp.getPermissions().containsKey(permission.substring(1))) { return !pp.getPermissions().get(permission.substring(1)); }
		} else {
			if (pp.getPermissions().containsKey(permission.substring(1))) { return pp.getPermissions().get(permission.substring(1)); }
		}
		return player.hasPermission(permission);
	}
	
	@Override
	public boolean playerInGroup(String x, OfflinePlayer player, String group) {
		return PermissionManager.getInstance().getPlayer(player.getUniqueId()).getSubgroups().contains(group);
	}
	
	@Override
	public boolean playerAddGroup(String x, OfflinePlayer player, String group) {
		PermissionManager pm = PermissionManager.getInstance();
		PlayerPerms pp = pm.getPlayer(player.getUniqueId());
		if (pp.addSubgroup(group)) {
			pm.setPlayer(player.getUniqueId(), pp);
			return true;
		} else return false;
	}
	
	@Override
	public boolean playerRemoveGroup(String x, OfflinePlayer player, String group) {
		PermissionManager pm = PermissionManager.getInstance();
		PlayerPerms pp = pm.getPlayer(player.getUniqueId());
		if (pp.removeSubgroup(group)) {
			pm.setPlayer(player.getUniqueId(), pp);
			return true;
		} else return false;
	}
	
	@Override
	public String[] getPlayerGroups(String x, OfflinePlayer player) {
		return toArray(PermissionManager.getInstance().getPlayer(player.getUniqueId()).getSubgroups());
	}
	
	@Override
	public String getPrimaryGroup(String x, OfflinePlayer player) {
		return PermissionManager.getInstance().getPlayer(player.getUniqueId()).getGroup();
	}
	
	@Override
	public boolean groupHas(String x, String group, String permission) {
		PermissionManager pm = PermissionManager.getInstance();
		if (pm.containsGroup(group)) {
			if(pm.getGroup(group).hasPermission("*")) return true;
			else if (pm.getGroup(group).hasPermission(permission)) return true;
		}
		return false;
	}
	
	@Override
	public boolean groupAdd(String x, String group, String permission) {
		PermissionManager pm = PermissionManager.getInstance();
		if (pm.containsGroup(group)) {
			GroupPerms gp = pm.getGroup(group);
			if (permission.startsWith("-")) {
				if (gp.addPermission(permission.substring(1), false)) {
					pm.setGroup(group, gp);
					return true;
				}
			} else {
				if (gp.addPermission(permission, false)) {
					pm.setGroup(group, gp);
					return true;
				}
			}
		}
		return false;
	}
	
	@Override
	public boolean groupRemove(String x, String group, String permission) {
		PermissionManager pm = PermissionManager.getInstance();
		if (pm.removeGroup(group)) { return true; }
		return false;
	}
	
	@Override
	public String[] getGroups() {
		return toArray(PermissionManager.getInstance().getGroups());
	}
	
	private String[] toArray(Collection<String> strings) {
		return strings.toArray(new String[0]);
	}
	
	@Override
	public boolean playerHas(String world, String player, String permission) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerHas(world, offlinePlayer, permission);
	}
	
	@Override
	public boolean playerAdd(String world, String player, String permission) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerAdd(world, offlinePlayer, permission);
	}
	
	@Override
	public boolean playerRemove(String world, String player, String permission) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerRemove(world, offlinePlayer, permission);
	}
	
	@Override
	public boolean playerInGroup(String world, String player, String group) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerInGroup(world, offlinePlayer, group);
	}
	
	@Override
	public boolean playerAddGroup(String world, String player, String group) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerAddGroup(world, offlinePlayer, group);
	}
	
	@Override
	public boolean playerRemoveGroup(String world, String player, String group) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return false;
		return playerRemoveGroup(world, offlinePlayer, group);
	}
	
	@Override
	public String[] getPlayerGroups(String world, String player) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return new String[] { PermissionManager.getInstance().getDefaultGroup() };
		return getPlayerGroups(world, offlinePlayer);
	}
	
	@Override
	public String getPrimaryGroup(String world, String player) {
		OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(player);
		if (offlinePlayer == null) return PermissionManager.getInstance().getDefaultGroup();
		return getPrimaryGroup(world, offlinePlayer);
	}
}
