package net.maunium.bukkit.MauShops;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.configuration.serialization.ConfigurationSerializable;
import org.bukkit.configuration.serialization.SerializableAs;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

@SerializableAs(value = "MauShop")
public class Shop implements ConfigurationSerializable {
	private List<ItemStack[]> pages;
	private String name;
	
	public Shop(String name) {
		pages = new ArrayList<ItemStack[]>();
		this.name = name;
	}
	
	public Inventory asInventory(int page) throws IndexOutOfBoundsException {
		if (pages.size() > 0 && pages.size() >= page) {
			Inventory i = Bukkit.createInventory(null, 5, name);
			i.setContents(pages.get(page - 1));
			int x = 4 * 9 - 1;
			
			i.setItem(x + 5, getItemName());
			if (pages.size() > 1) if (page == 1) {
				i.setItem(x + 9, getItemNextPage(true));
				i.setItem(x + 1, getItemPrevPage(false));
			} else if (page == pages.size()) {
				i.setItem(x + 9, getItemNextPage(false));
				i.setItem(x + 1, getItemPrevPage(true));
			} else {
				i.setItem(x + 9, getItemNextPage(true));
				i.setItem(x + 1, getItemPrevPage(true));
			}
			return i;
		} else throw new IndexOutOfBoundsException("Page number can't be larger than amount of pages.");
		
	}
	
	private ItemStack getItemName() {
		ItemStack name = new ItemStack(Material.NAME_TAG);
		ItemMeta im = name.getItemMeta();
		im.setDisplayName(ChatColor.GREEN + this.name);
		name.setItemMeta(im);
		return name;
	}
	
	private ItemStack getItemNextPage(boolean enabled) {
		Material m;
		if (enabled) m = Material.EYE_OF_ENDER;
		else m = Material.ENDER_PEARL;
		ItemStack next = new ItemStack(m);
		ItemMeta im = next.getItemMeta();
		im.setDisplayName(ChatColor.AQUA + "Next Page");
		next.setItemMeta(im);
		return next;
	}
	
	private ItemStack getItemPrevPage(boolean enabled) {
		Material m;
		if (enabled) m = Material.EYE_OF_ENDER;
		else m = Material.ENDER_PEARL;
		ItemStack prev = new ItemStack(m);
		ItemMeta im = prev.getItemMeta();
		im.setDisplayName(ChatColor.AQUA + "Previous Page");
		prev.setItemMeta(im);
		return prev;
	}
	
	@Override
	public Map<String, Object> serialize() {
		Map<String, Object> rtrn = new HashMap<String, Object>();
		rtrn.put("name", name);
		rtrn.put("page-count", pages.size());
		int page = 1;
		for (ItemStack[] pageArray : pages) {
			List<ItemStack> pageContents = Arrays.asList(pageArray);
			rtrn.put("page-" + page, pageContents);
			page++;
		}
		return null;
	}
	
	public Shop(Map<String, Object> serialized) {
		this.name = (String) serialized.get("name");
		int pageCount = Integer.parseInt(serialized.get("page-count").toString());
		//List<ItemStack[]> pages = new ArrayList<ItemStack[]>(pageCount);
		PageLoop: for (int i = 0; i < pageCount; i++) {
			Object oo = serialized.get("page-" + (i + 1));
			if (!(oo instanceof List<?>)) continue PageLoop;
			List<?> list = (List<?>) oo;
			List<ItemStack> pageContents = new ArrayList<ItemStack>();
			ContentLoop: for (Object o : list) {
				if (!(o instanceof ItemStack)) continue ContentLoop;
				pageContents.add((ItemStack) o);
			}
		}
	}
	
	public Shop deserialize(Map<String, Object> serialized) {
		return new Shop(serialized);
	}
	
	public Shop valueOf(Map<String, Object> serialized) {
		return new Shop(serialized);
	}
}
