package net.maunium.bukkit.MauSongPlayer;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;

public class MauSong {
	private HashMap<Integer, MauLayer> layerHashMap = new HashMap<Integer, MauLayer>();
	private short songHeight, length;
	private String title;
	private File path;
	private String author, description;
	private float speed, delay;
	
	public MauSong(MauSong other) {
		this.speed = other.getSpeed();
		delay = 20 / speed;
		this.layerHashMap = other.getLayerHashMap();
		this.songHeight = other.getSongHeight();
		this.length = other.getLength();
		this.title = other.getTitle();
		this.author = other.getAuthor();
		this.description = other.getDescription();
		this.path = other.getPath();
	}
	
	public MauSong(float speed, HashMap<Integer, MauLayer> layerHashMap, short songHeight, final short length, String title, String author, String description, File path) {
		this.speed = speed;
		delay = 20 / speed;
		this.layerHashMap = layerHashMap;
		this.songHeight = songHeight;
		this.length = length;
		this.title = title;
		this.author = author;
		this.description = description;
		this.path = path;
	}
	
	public HashMap<Integer, MauLayer> getLayerHashMap() {
		return layerHashMap;
	}
	
	public short getSongHeight() {
		return songHeight;
	}
	
	public short getLength() {
		return length;
	}
	
	public String getTitle() {
		return title;
	}
	
	public String getAuthor() {
		return author;
	}
	
	public File getPath() {
		return path;
	}
	
	public String getDescription() {
		return description;
	}
	
	public float getSpeed() {
		return speed;
	}
	
	public float getDelay() {
		return delay;
	}
	
	public static MauSong read(File decodeFile) {
		HashMap<Integer, MauLayer> layerHashMap = new HashMap<Integer, MauLayer>();
		try {
			DataInputStream dis = new DataInputStream(new FileInputStream(decodeFile));
			short length = readShort(dis);
			short songHeight = readShort(dis);
			String title = readString(dis);
			String author = readString(dis);
			readString(dis);
			String description = readString(dis);
			float speed = readShort(dis) / 100f;
			dis.readBoolean(); // auto-save
			dis.readByte(); // auto-save duration
			dis.readByte(); // x/4ths, time signature
			readInt(dis); // minutes spent on project
			readInt(dis); // left clicks (why?)
			readInt(dis); // right clicks (why?)
			readInt(dis); // blocks added
			readInt(dis); // blocks removed
			readString(dis); // .mid/.schematic file name
			short tick = -1;
			while (true) {
				short jumpTicks = readShort(dis);
				if (jumpTicks == 0) {
					break;
				}
				tick += jumpTicks;
				short layer = -1;
				while (true) {
					short jumpLayers = readShort(dis);
					if (jumpLayers == 0) {
						break;
					}
					layer += jumpLayers;
					setNote(layer, tick, /* Instrument */dis.readByte(), /* Note */dis.readByte(), layerHashMap);
				}
			}
			for (int i = 0; i < songHeight; i++) {
				MauLayer l = layerHashMap.get(i);
				if (l != null) {
					l.setName(readString(dis));
					l.setVolume(dis.readByte());
				}
			}
			return new MauSong(speed, layerHashMap, songHeight, length, title, author, description, decodeFile);
		} catch (Throwable t) {
			t.printStackTrace();
		}
		return null;
	}
	
	private static void setNote(int layer, int ticks, byte instrument, byte key, HashMap<Integer, MauLayer> layerHashMap) {
		MauLayer l = layerHashMap.get(layer);
		if (l == null) {
			l = new MauLayer();
			layerHashMap.put(layer, l);
		}
		l.setNote(ticks, new MauNote(instrument, key));
	}
	
	private static short readShort(DataInputStream dis) throws IOException {
		int byte1 = dis.readUnsignedByte();
		int byte2 = dis.readUnsignedByte();
		return (short) (byte1 + (byte2 << 8));
	}
	
	private static int readInt(DataInputStream dis) throws IOException {
		int byte1 = dis.readUnsignedByte();
		int byte2 = dis.readUnsignedByte();
		int byte3 = dis.readUnsignedByte();
		int byte4 = dis.readUnsignedByte();
		return byte1 + (byte2 << 8) + (byte3 << 16) + (byte4 << 24);
	}
	
	private static String readString(DataInputStream dis) throws IOException {
		int length = readInt(dis);
		StringBuilder sb = new StringBuilder(length);
		for (; length > 0; --length) {
			char c = (char) dis.readByte();
			if (c == (char) 0x0D) {
				c = ' ';
			}
			sb.append(c);
		}
		return sb.toString();
	}
}
