package net.maunium.bukkit.MauTeams;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;
import java.util.UUID;

import org.bukkit.ChatColor;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.configuration.serialization.ConfigurationSerialization;
import org.bukkit.entity.Player;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.plugin.java.JavaPlugin;

import net.maunium.bukkit.MauBukLib.I18n;
import net.maunium.bukkit.MauBukLib.I18n.Internationalizer;
import net.maunium.bukkit.MauBukLib.MauUtils;

public class MauTeams extends JavaPlugin implements Internationalizer {
	
	public String version;
	public final String tping = "MauTeamsTeleporting";
	private I18n i18n;
	public final String name = "MauTeams", author = "Tulir293", stag = ChatColor.DARK_GREEN + "[" + ChatColor.GREEN + name + ChatColor.DARK_GREEN + "] " + ChatColor.GRAY, errtag = ChatColor.DARK_RED
			+ "[" + ChatColor.RED + name + ChatColor.DARK_RED + "] " + ChatColor.RED;
	private final String teamMeta = "MauCurrentTeam", chatMeta = "MauIsTeamChat";
	private HashMap<String, Team> teams = new HashMap<String, Team>();
	
	@Override
	public void onEnable() {
		long st = System.currentTimeMillis();
		version = this.getDescription().getVersion();
		this.saveDefaultConfig();
		
		Team.setPlugin(this);
		
		this.saveResource("en_US.lang", true);
		this.saveResource("fi_FI.lang", true);
		try {
			i18n = I18n.createInstance(this.getDataFolder(), getConfig().getString("language"));
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		
		ConfigurationSerialization.registerClass(Team.class);
		teams = new HashMap<String, Team>();
		for (String s : getConfig().getStringList("teams")) {
			YamlConfiguration conf = new YamlConfiguration();
			try {
				conf.load(new File(this.getDataFolder() + File.separator + "teams" + File.separator + s + ".yml"));
			} catch (Exception e) {
				getLogger().severe("Failed to load team " + s);
				e.printStackTrace();
				continue;
			}
			Object o = conf.get("value");
			if (o != null && o instanceof Team) teams.put(s, (Team) o);
			else getLogger().severe("Failed to load team " + s);
		}
		this.getServer().getPluginManager().registerEvents(new JoinListener(this), this);
		this.getServer().getPluginManager().registerEvents(new PlayerPvPListener(this), this);
		this.getServer().getPluginManager().registerEvents(new TPListener(this), this);
		this.getServer().getPluginManager().registerEvents(new ChatListener(this), this);
		this.getCommand("mauteams").setExecutor(new CommandTeam(this));
		
		for (Player p : this.getServer().getOnlinePlayers())
			if (!isInTeam(p)) {
				Team t = getTeamByUUID(p.getUniqueId());
				if (t != null) addToTeam(p, t);
			}
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " enabled in " + et + "ms.");
	}
	
	@Override
	public void onDisable() {
		long st = System.currentTimeMillis();
		
		List<String> tms = new ArrayList<String>();
		tms.addAll(teams.keySet());
		getConfig().set("teams", tms);
		
		for (Entry<String, Team> e : teams.entrySet()) {
			YamlConfiguration conf = new YamlConfiguration();
			conf.set("value", e.getValue());
			try {
				conf.save(new File(this.getDataFolder() + File.separator + "teams" + File.separator + e.getKey() + ".yml"));
			} catch (IOException ex) {
				getLogger().severe("Failed to save team " + e.getKey());
				ex.printStackTrace();
				continue;
			}
		}
		
		this.saveConfig();
		
		int et = (int) (System.currentTimeMillis() - st);
		getLogger().info(name + " v" + version + " by " + author + " disabled in " + et + "ms.");
	}
	
	public Set<Entry<String, Team>> getTeams() {
		return teams.entrySet();
	}
	
	/**
	 * Remove the given team.
	 */
	public void removeTeam(Team t) {
		teams.values().remove(t);
		for (UUID u : t.getMembers()) {
			Player p = getServer().getPlayer(u);
			if (p != null) removeFromTeam(p);
		}
		for (UUID u : t.getManagers()) {
			Player p = getServer().getPlayer(u);
			if (p != null) removeFromTeam(p);
		}
		Player p = getServer().getPlayer(t.getOwner());
		if (p != null) removeFromTeam(p);
		new File(this.getDataFolder() + File.separator + "teams" + File.separator + t.getName().toLowerCase() + ".yml").delete();
	}
	
	public boolean hasTeam(String team) {
		return teams.containsKey(team.toLowerCase());
	}
	
	/**
	 * Add a team with the given owner and name.
	 */
	public void addTeam(Player p, String team) {
		Team t = new Team(team, p.getUniqueId());
		teams.put(team.toLowerCase(), t);
		addToTeam(p, t);
	}
	
	/**
	 * Adds the team metadata tag for a player.
	 */
	public void addToTeam(Player p, Team team) {
		Team t = getTeamByPlayer(p);
		if (t != null) t.remove(p);
		p.setMetadata(teamMeta, new FixedMetadataValue(this, team));
	}
	
	/**
	 * Checks if a player has the team metadata tag (and thus if the player is in a team)
	 * 
	 * @return True if the player is in a team, false if not.
	 */
	public boolean isInTeam(Player p) {
		return getTeamByPlayer(p) != null;
	}
	
	/**
	 * Removes the team metadata tag from a player
	 */
	public void removeFromTeam(Player p) {
		if (isInTeam(p)) p.removeMetadata(teamMeta, this);
		if (isInTeamChat(p)) removeFromTeamChat(p);
	}
	
	/**
	 * Check if the player is in a team chat.
	 * 
	 * @return True if the player is in team chat, false if not.
	 */
	public boolean isInTeamChat(Player p) {
		return p.hasMetadata(chatMeta);
	}
	
	/**
	 * Add the team chat metadata to a player.
	 */
	public void addToTeamChat(Player p) {
		p.setMetadata(chatMeta, new FixedMetadataValue(this, true));
	}
	
	/**
	 * Remove the team chat metadata from a player.
	 */
	public void removeFromTeamChat(Player p) {
		p.removeMetadata(chatMeta, this);
	}
	
	/**
	 * Get a team from the metadata of a player.
	 * 
	 * @return The team of the player, or null if metadata not found.
	 */
	public Team getTeamByPlayer(Player p) {
		if (p.hasMetadata(teamMeta)) {
			MetadataValue mv = MauUtils.getMetadata(p, teamMeta, this);
			if (mv != null && mv.value() != null && mv.value() instanceof Team) return (Team) mv.value();
			else return null;
		} else return null;
	}
	
	/**
	 * Get a team by it's name.
	 * 
	 * @return As specified by {@link HashMap#get(Object)}
	 */
	public Team getTeamByName(String name) {
		return teams.get(name.toLowerCase());
	}
	
	/**
	 * Searches for a team that the specified UUID is a member of.
	 * 
	 * @return The team, or null if not found.
	 */
	public Team getTeamByUUID(UUID player) {
		for (Team t : teams.values())
			if (t.isPartOfTeam(player)) return t;
		return null;
	}
	
	@Override
	public String format(String node, Object... replace) {
		return i18n.format(node, replace);
	}
}