package net.maunium.bukkit.MauTracker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

import net.maunium.bukkit.MauBukLib.MauBukLib;

public class CommandTrack implements CommandExecutor {
	private MauTracker plugin;
	
	public CommandTrack(MauTracker plugin) {
		this.plugin = plugin;
	}
	
	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label, String[] args) {
		Block mid;
		if (sender instanceof Player) mid = ((Player) sender).getLocation().getBlock().getRelative(BlockFace.DOWN);
		else {
			sender.sendMessage(plugin.errtag + plugin.format("error.onlyingame", label));
			return true;
		}
		Player p = (Player) sender;
		if(!p.hasPermission("mautracker.track")){
			sender.sendMessage(plugin.errtag + plugin.format("error.permission"));
			return true;
		}
		Player igo = null;
		if (args.length > 0) {
			igo = MauBukLib.getUtils().getPlayer(args[0]);
		}
		if (mid.getType().equals(Material.DIAMOND_BLOCK)) {
			track(p, mid, Material.OBSIDIAN, Material.GOLD_BLOCK, 7000, igo);
			return true;
		} else if (mid.getType().equals(Material.GOLD_BLOCK)) {
			track(p, mid, Material.COAL_BLOCK, Material.IRON_BLOCK, 350, igo);
			return true;
		} else {
			p.sendMessage(plugin.errtag + plugin.format("error.notinmiddle"));
			return true;
		}
	}
	
	public void track(Player p, Block mid, Material arm, Material head, int max, Player ignoreOthers) {
		int north = getLength(mid, arm, head, BlockFace.NORTH);
		if (north == -1) {
			p.sendMessage(plugin.errtag + plugin.format("error.notvalidend", "North"));
			return;
		} else if (north > max) {
			north = max;
		}
		int east = getLength(mid, arm, head, BlockFace.EAST);
		if (east == -1) {
			p.sendMessage(plugin.errtag + plugin.format("error.notvalidend", "East"));
			return;
		} else if (east > max) {
			east = max;
		}
		int south = getLength(mid, arm, head, BlockFace.SOUTH);
		if (south == -1) {
			p.sendMessage(plugin.errtag + plugin.format("error.notvalidend", "South"));
			return;
		} else if (south > max) {
			south = max;
		}
		int west = getLength(mid, arm, head, BlockFace.WEST);
		if (west == -1) {
			p.sendMessage(plugin.errtag + plugin.format("error.notvalidend", "West"));
			return;
		} else if (west > max) {
			west = max;
		}
		Map<Location, Material> blocks = new HashMap<Location, Material>();
		for (Player p2 : getPlayers(mid.getLocation(), north, east, south, west)) {
			if (p2.getUniqueId().equals(p.getUniqueId())) continue;
			if (ignoreOthers != null) if (!p2.getUniqueId().equals(ignoreOthers.getUniqueId())) continue;
			
			int x = Math.round((p2.getLocation().getBlockX() - mid.getLocation().getBlockX()) / 25);
			int z = Math.round((p2.getLocation().getBlockZ() - mid.getLocation().getBlockZ()) / 25);
			p.sendMessage(plugin.stag + plugin.format("message.playerloc", p2.getName(), x, z, x * 25, z * 25));
			Block b = mid.getLocation().add(x, 1, z).getBlock();
			if (!blocks.containsKey(b.getLocation())) blocks.put(b.getLocation(), Material.getMaterial(b.getType().toString()));
			b.setType(Material.QUARTZ_BLOCK);
		}
		if (!blocks.isEmpty()) {
			Bukkit.getScheduler().runTaskLater(plugin, new TrackThread(blocks), 100);
			p.sendMessage(plugin.stag + plugin.format("message.complete"));
		} else p.sendMessage(plugin.stag + plugin.format("message.noplayers"));
	}
	
	private class TrackThread implements Runnable {
		private Map<Location, Material> blocks;
		
		public TrackThread(Map<Location, Material> blocks) {
			this.blocks = blocks;
		}
		
		@Override
		public void run() {
			for (Entry<Location, Material> e : blocks.entrySet()) {
				Location l = e.getKey();
				l.getBlock().setType(e.getValue());
			}
		}
	}
	
	private List<Player> getPlayers(Location l, int north, int east, int south, int west) {
		List<Player> rtrn = new ArrayList<Player>();
		for (Player p : plugin.getServer().getOnlinePlayers()) {
			int x = p.getLocation().getBlockX();
			int z = p.getLocation().getBlockZ();
			if (z > l.getBlockZ() - north && z < l.getBlockZ() + south && x > l.getBlockX() - west && x < l.getBlockX() + east) {
				rtrn.add(p);
			}
		}
		return rtrn;
	}
	
	private int getLength(Block mid, Material arm, Material end, BlockFace direction) {
		int length = 25;
		mid = mid.getRelative(direction);
		while (mid.getType().equals(arm)) {
			length += 25;
			mid = mid.getRelative(direction);
		}
		if (!mid.getType().equals(end)) { return -1; }
		return length;
	}
}
